#include "Conversion.hpp"

namespace decomposition
{

	Conversion::Conversion(vector<vector<double>>* inputSource, EnergySystem* enrg): input{inputSource}, linkToSystem{enrg}, numScen{enrg->getNumScen()}, numTimeSteps{enrg->getNumTimeSteps()}
	{
		output.resize(numTimeSteps);
		produced.resize(numTimeSteps);
		exported.resize(numTimeSteps);

		designVarDisc.resize(numScen);

		for(int t=0;t<numTimeSteps;++t)
		{
			output[t].resize(numScen);
			produced[t].resize(numScen);
			exported[t].resize(numScen);
		}
	};

	void Conversion::setCostFunction(double unitCost, double maxCap, int numSizeIntervals,int CRFtime, string type, bool VSS=false, double EVPsize=0)
	{
		lifeTime=CRFtime;
		convTech=type;
		char* name=&type[0];
		baseCost=unitCost;
		capN=maxCap;
		CRF=r*pow(1+r,lifeTime)/(pow(1+r,lifeTime)-1);
		cout<<"CRF for "<<type<< " is :"<<CRF<<endl;
		if(!VSS)
		{
			numDiscrete=numSizeIntervals;
			sizes.resize(numDiscrete);
			designVarBin.resize(numDiscrete);

			for(int n=0;n<numDiscrete;++n)
			{
				sizes[n]=n*capN/(numDiscrete-1);
				sprintf(linkToSystem->clabel, "%s[%d]", name, n+1);
				designVarBin[n].setIndependentVariable(linkToSystem->addVariable(), compgraph::BINARY, I(0,1),0.,-1,linkToSystem->clabel);
			}
		}
		else
		{
			numDiscrete=1;
			sizes.resize(numDiscrete);
			designVarBin.resize(numDiscrete);
			sizes[numDiscrete-1]=EVPsize;
			sprintf(linkToSystem->clabel, "%s[%d]", name, numDiscrete);
			designVarBin[numDiscrete-1].setIndependentVariable(linkToSystem->addVariable(), compgraph::BINARY, I(0,1),0.,-1,linkToSystem->clabel);
		}


	};

	void Conversion::setConversionFunction(double constA=1.0, double constB=0.0)
	{
		a=constA;
		b=constB; 
		string varName1=convTech+"produced";
		char* name1=&varName1[0];

		int t=0;
		for(vector<vector<double>>::iterator itRow=input->begin();itRow!=input->end();++itRow)
		{
			int h=0;
			for(vector<double>::iterator itCol=itRow->begin();itCol!=itRow->end();++itCol)
			{
				sprintf(linkToSystem->clabel,"%s[%d][%d]",name1,t+1,h+1);
				produced[t][h].setIndependentVariable(linkToSystem->addVariable(),compgraph::CONTINUOUS,I(0,1000),0.,h+1,linkToSystem->clabel);
				if((*itCol)>0)
				{
					output[t][h]=a*(*itCol)+b;
				}
				else
				{
					output[t][h]=0;
				}
				cout<<output[t][h]<<endl;
				h++;
			}
			t++;
		};

		string varName2=convTech+"zDisc";
		char* name2=&varName2[0];
		for(int h=0;h<numScen;++h)
		{
			sprintf(linkToSystem->clabel,"%s[%d]",name2,h+1);
			designVarDisc[h].setIndependentVariable(linkToSystem->addVariable(),compgraph::CONTINUOUS,I(0,capN),0.,h+1,linkToSystem->clabel);
		}
	};

	void Conversion::optimizeCapacity()
	{
		designConBin=-1;
		designConDisc.resize(numScen);

		for(int n=0;n<numDiscrete;++n)
		{
			designConBin+=designVarBin[n];
		}
		designConBin.setDependentVariable(linkToSystem->addConstraint(),compgraph::EQUALITY,false,-1);

		for(int h=0;h<numScen;++h)
		{
			designConDisc[h]=designVarDisc[h];
			for(int n=0;n<numDiscrete;++n)
			{
				designConDisc[h]-=designVarBin[n]*sizes[n];
			}
			designConDisc[h].setDependentVariable(linkToSystem->addConstraint(),compgraph::EQUALITY,true,h+1);
		}

		for(int t=0;t<numTimeSteps;++t)
		{
			for(int h=0;h<numScen;++h)
			{
				exported[t][h]=produced[t][h];
				exported[t][h]-=output[t][h]*designVarDisc[h];
				exported[t][h].setDependentVariable(linkToSystem->addConstraint(),compgraph::EQUALITY,true,h+1);
			};
		}
	};

	void Conversion::addCAPEX(vector<Objective>& objectiveFunction, vector<Constraints>& budget)
	{
		for(int h=0;h<numScen;++h)
		{

			objectiveFunction[h]+=baseCost*designVarDisc[h]*CRF*(1+maintFac);
			budget[h]+=baseCost*designVarDisc[h];

		}

	};
		
}