//Conversion class

#ifndef CONVERSION_H
#define CONVERSION_H
#include "headerFiles.hpp"
#include "PrimarySource.hpp"
#include "EnergySystem.hpp"




namespace decomposition
{


class Conversion
{
private:


	vector<vector<double>>* input; //pointer to input vales from primary source/energy source, e.g solar irradiation etc

	EnergySystem* linkToSystem;

	double a, b; //parameter values for linear or affine linear conversion equation

	int numScen, numTimeSteps, numDiscrete, lifeTime; 

	string convTech;

	double baseCost, CRF,r{0.06},capN{100}, econScaleFac{0.7}, maintFac{0.05}; 
	//cost0=base cost, cap0=base capacity, capN=max capacity
	//econScaleFaf= economies of scale factor, eff=efficiency

	vector<double> sizes; //vector with numDiscrete size intervals for conversion technology

	vector<vector<double>> output; //vector with output values in unit energy MW, will be initialized in setConversionFunction

	vector<Variables> designVarBin;
	Constraints designConBin;

	vector<Variables> designVarDisc;
	vector<Constraints> designConDisc;

	vector<vector<Variables>> produced;
	vector<vector<Constraints>> exported;

public:
	friend EnergySystem;

	string unitName;

	Conversion(vector<vector<double>>* inputSource,EnergySystem* enrg); //constructor

	Conversion* getPointer(){return this;}; 

	void setCostFunction(double unitCost, double maxCap, int numSizeIntervals, int CRFtime, string type,bool VSS=false, double EVPsize=0); //set CAPEX cost function, dependent on installed size

	void setConversionFunction(double constA=1.0, double constB=0.0); //set output conversion function, how much energy produced

	vector<vector<Variables>>* getOutput(){return &produced;}; //returns output value for timestep t and scenario h, only one double value

	void optimizeCapacity();

	void addCAPEX(vector<Constraints>& objectiveFunction, vector<Constraints>& budget);
	
};

}

#endif