#include "EndUser.hpp"


namespace decomposition
{

	EndUser::EndUser(EnergySystem* enrg): linkToSystem{enrg}, numScen{enrg->getNumScen()}, numTimeSteps{enrg->getNumTimeSteps()}
	{};
	
	void EndUser::addUtilityDemand(Utility* utilityType, vector<vector<double>>* input)
	{
		map<Utility*,vector<vector<Constraints>>>::iterator it1=demandMap.find(utilityType);
		if(it1!=demandMap.end())
		{
			vector<vector<double>> temp(numTimeSteps,vector<double>(numScen));

			int t=0;
			for(vector<vector<double>>::iterator itRow=input->begin();itRow!=input->end();++itRow)
			{
				int h=0;
				for(vector<double>::iterator itCol=itRow->begin();itCol!=itRow->end();++itCol)
				{
					temp[t][h]=(*itCol);
					h++;
				}
				t++;
			};
			cout<<"Added utility user demand to exisiting utility type "<<utilityType->name<<"."<<endl;
			for(int t=0;t<numTimeSteps;++t)
			{
				for(int h=0;h<numScen;++h)
				{
					it1->second[t][h]+=temp[t][h];
				}
			}
		}
		else
		{
			vector<vector<Constraints>> temp2(numTimeSteps,vector<Constraints>(numScen));

			int t=0;
			for(vector<vector<double>>::iterator itRow=input->begin();itRow!=input->end();++itRow)
			{
				int h=0;
				for(vector<double>::iterator itCol=itRow->begin();itCol!=itRow->end();++itCol)
				{
					temp2[t][h]=(*itCol);
					h++;
				}
				t++;
			};
			importUtilities.push_back(utilityType);
			demandMap.insert(pair<Utility*,vector<vector<Constraints>>>(utilityType,temp2));
			cout<<"Added user demand and import price for new utility type "<<utilityType->name<<"."<<endl;
		};

		
	}


	void EndUser::addUtilitySource(Utility* utilityType)
	{	
		vector<Utility*>::iterator it=find(inputUtilities.begin(),inputUtilities.end(),utilityType);
		if(it==inputUtilities.end())
		{
			inputUtilities.push_back(utilityType);
			cout<<"Added utility type "<<utilityType->name<<"."<<endl;
		}
		else
		{
			cout<<"Error, utility already added. Check system.\n";

		};
	};

	void EndUser::setUtilityImportConstraint(Utility* utilityType)
	{
		map<Utility*,vector<vector<Constraints>>>::iterator it1=demandMap.find(utilityType);
		if(it1!=demandMap.end())
		{
			cout<<"Found user demand for the requested utilitytype "<<utilityType->name<<"."<<endl;
			vector<Utility*>::iterator it3=find(inputUtilities.begin(),inputUtilities.end(),utilityType);
			vector<Utility*>::iterator it4=find(importUtilities.begin(),importUtilities.end(),utilityType);
			if(it3!=inputUtilities.end() && it4!=importUtilities.end())
			{
				for(int t=0;t<numTimeSteps;++t)
					{
						for(int h=0;h<numScen;++h)
						{
							it1->second[t][h]-=(*it3)->getExport(t,h);
							it1->second[t][h]-=(*it4)->getImport(t,h);
						}
					}
			}
			for(int t=0;t<numTimeSteps;++t)
			{
				for(int h=0; h<numScen;++h)
				{
					it1->second[t][h].setDependentVariable(linkToSystem->addConstraint(),compgraph::EQUALITY,true,h+1);
				}
			}
		}
		else
		{
			cout<<"Could not find any demand constraint for the requested utility type "<<utilityType->name<<"."<<endl;
		};
	};


	void EndUser::getUtilityExport(vector<Objective>& objectiveFunction)
	{	
		vector<Utility*>::iterator it;
		for(it=inputUtilities.begin();it!=inputUtilities.end();++it)
		{
			double FiT=(*it)->constFiT;
			double FitExtra=((*it)->surplusFiT)*FiT;
			for(int t=0;t<numTimeSteps;++t)
			{
				for(int h=0;h<numScen;++h)
				{
					objectiveFunction[h]-=(*it)->getExport(t,h)*FiT*365;
					objectiveFunction[h]-=(*it)->getSurplus(t,h)*FitExtra*365;
				}
			}
		}
		
	};

	void EndUser::getUtilityImport(vector<Objective>& objectiveFunction)
	{
		vector<Utility*>::iterator it;
		for(it=importUtilities.begin();it!=importUtilities.end();++it)
		{
			for(int t=0;t<numTimeSteps;++t)
			{
				for(int h=0;h<numScen;++h)
				{
					objectiveFunction[h]+=(*it)->getImport(t,h)*(*it)->importPrice[t][h]*365;
				}
			}
		}

	};

}