#include "Utility.hpp"

namespace decomposition
{

Utility::Utility(vector<vector<double>>* price,EnergySystem* enrg, string utilityName): linkToSystem{enrg}, numScen{enrg->getNumScen()}, numTimeSteps{enrg->getNumTimeSteps()}, name{utilityName}
{
	string varName1=utilityName+"exported";
	string varName2=utilityName+"imported";
	string varname3=utilityName+"surplus";
	char* name1=&varName1[0];
	char* name2=&varName2[0];
	char* name3=&varname3[0];

	exported.resize(numTimeSteps);
	imported.resize(numTimeSteps);
	surplus.resize(numTimeSteps);

	producedCon.resize(numTimeSteps);
	importPrice.resize(numTimeSteps);

	for(int t=0;t<numTimeSteps;++t)
	{
		exported[t].resize(numScen);
		imported[t].resize(numScen);
		surplus[t].resize(numScen);

		producedCon[t].resize(numScen);
		importPrice[t].resize(numScen);

		for(int h=0;h<numScen;++h)
		{
			sprintf(linkToSystem->clabel,"%s[%d][%d]",name1,t+1,h+1);
			exported[t][h].setIndependentVariable(linkToSystem->addVariable(),compgraph::CONTINUOUS,I(0,1000),0.,h+1,linkToSystem->clabel);
			sprintf(linkToSystem->clabel,"%s[%d][%d]",name2,t+1,h+1);
			imported[t][h].setIndependentVariable(linkToSystem->addVariable(),compgraph::CONTINUOUS,I(0,1000),0.,h+1,linkToSystem->clabel);
			sprintf(linkToSystem->clabel,"%s[%d][%d]",name3,t+1,h+1);
			surplus[t][h].setIndependentVariable(linkToSystem->addVariable(),compgraph::CONTINUOUS,I(0,1000),0.,h+1,linkToSystem->clabel);
		}
	};

	cout<<"Initialized utility object "<<name<<".\n";
	uncertainPrice=true;
	vector<vector<double>>::iterator it1;
	int t=0;
	for(it1=price->begin();it1!=price->end();++it1)
	{
		vector<double>::iterator it2;
		int h=0;
		for(it2=it1->begin();it2!=it1->end();++it2)
		{
			importPrice[t][h]=(*it2);
			++h;
		}
		++t;
	};
	cout<<"Imported import price for utility "<<utilityName<<".\n";


};

void Utility::addUtility(vector<vector<Variables>>* input, double fraction)
{
	inputSources.push_back(input);
	inputFractions.push_back(fraction);

	cout<<"Added utility input to "<<name<<".\n";
	int t=0;
	for(vector<vector<Variables>>::iterator itRow=input->begin();itRow!=input->end();++itRow)
	{
		int h=0;
		for(vector<Variables>::iterator itCol=itRow->begin();itCol!=itRow->end();++itCol)
		{
			if(inputSources.size()<=1)
			{
				producedCon[t][h]=0;
				producedCon[t][h]+=(*itCol)*inputFractions[inputFractions.size()-1];
			}
			else
			{
				producedCon[t][h]+=(*itCol)*inputFractions[inputFractions.size()-1];
			}
			h++;
		}
		t++;
	};

};

void Utility::setUtilityExportConstraint()
{
	for(int t=0;t<numTimeSteps;++t)
	{
		for(int h=0;h<numScen;++h)
		{
			producedCon[t][h]-=exported[t][h];
			producedCon[t][h]-=surplus[t][h];
			producedCon[t][h].setDependentVariable(linkToSystem->addConstraint(),compgraph::EQUALITY,true,h+1);
		}
	}
};	

}